@@ -1,11 +1,13 @@
 """
 a Python NATS client
 
->>> from goingnats import Client
+>>> from goingnats import Client, one
 
 Check if __name__ == "__main__" for full example
 """
+__version__ = "2021.3.5"
 
+import json
 import queue
 import socket
 import threading
@@ -15,11 +17,11 @@ from collections import namedtuple
 
 
 class Client:
-    def __init__(self, *, name, host="0.0.0.0", port=4222):
+    def __init__(self, *, name, host="127.0.0.1", port=4222):
         self.host = host
         self.port = port
         self.name = name
-        self._buffer = b""
+        self.information = {}
         self._messages = queue.Queue()
         self._response = queue.Queue(maxsize=1)
         self._sid = 0
@@ -56,7 +58,7 @@ class Client:
                 f"can't connect to {self.host}:{self.port}"
             ) from e
         self._run = True
-        threading.Thread(target=self._thread, daemon=True).start()
+        threading.Thread(target=self._thread).start()
         return self
 
     def __exit__(self, exception_type, exception_value, exception_traceback):
@@ -72,60 +74,70 @@ class Client:
             self._sock.sendall(f"{string}\r\n".encode("utf-8"))
         except BrokenPipeError as e:
             raise BrokenPipeError("send failed") from e
+        except OSError as e:
+            # socket
+            if self._run:
+                raise OSError("send failed") from e
 
     def _thread(self):
         # https://docs.nats.io/nats-protocol/nats-protocol
+        buffer_ = b""
         while self._run:
             try:
                 received = self._sock.recv(4096)
             except ConnectionResetError as e:
                 raise ConnectionResetError("recv failed") from e
-            current, eom, next = received.partition(b"\r\n")
-            self._buffer += current
-            # End Of Message
-            if eom:
-                if self._buffer == b"PING":
+            except OSError as e:
+                if self._run:
+                    raise OSError("recv failed") from e
+
+            segments = received.split(b"\r\n")
+            if len(segments) > 1:
+                # buffer contains tail of previous received \r\ntail
+                segments[0] = buffer_ + segments[0]
+                # will be b"" if received end with \r\n
+                buffer_ = segments[-1]
+            else:
+                # no \r\n received nothing to do but buffer
+                buffer_ += received
+                continue
+
+            for segment in segments[:-1]:
+                if segment == b"PING":
                     self._send("PONG")
-                elif self._buffer == b"PONG":
+                elif segment == b"PONG":
                     pass
-                elif self._buffer == b"+OK":
+                elif segment == b"+OK":
                     pass
-                elif self._buffer.startswith(b"-ERR"):
-                    raise warnings.warn(self._buffer.decode("utf-8"))
-                elif self._buffer.startswith(b"INFO"):
-                    self._send(
-                        f'CONNECT {{"name": "{self.name}", "verbose": false}}'
-                    )
-                elif self._buffer.startswith(b"MSG"):
+                elif segment.startswith(b"-ERR"):
+                    warnings.warn(segment.decode("utf-8"))
+                elif segment.startswith(b"INFO"):
+                    self.information = json.loads(segment[4:])
+                    information = {
+                        "name": self.name,
+                        "verbose": False,
+                        "version": f"goingnats-{__version__}",
+                    }
+                    self._send(f"CONNECT {json.dumps(information)}")
+                elif segment.startswith(b"MSG"):
                     # MSG ...  \r\n[payload]\r\n
-                    split = self._buffer.split(b" ")
+                    split = segment.split(b" ")
                     subject = split[1]
                     # is it a request?
                     inbox = False
-                    if len(split) == 5:
+                    if len(split) >= 5:
                         inbox = split[3]
-                    # End Of Payload
-                    payload, eop, _ = next.partition(b"\r\n")
-                    if eop:
-                        # request
-                        if inbox:
-                            self._messages.put(Request(subject, inbox, payload))
-                        # response
-                        elif subject.startswith(b"INBOX."):
-                            self._response.put(Response(payload))
-                        # vanilla message
-                        else:
-                            self._messages.put(Message(subject, payload))
-                        next = b""
                 else:
-                    # payload over multiple recv calls
+                    # should be e a payload
+                    # request
                     if inbox:
-                        self._messages.put(Request(subject, inbox, payload))
+                        self._messages.put(Request(subject, inbox, segment))
+                    # response
                     elif subject.startswith(b"INBOX."):
-                        self._response.put(Response(payload))
+                        self._response.put(Response(segment))
+                    # vanilla message
                     else:
-                        self._messages.put(Message(subject, self._buffer))
-            self._buffer = next
+                        self._messages.put(Message(subject, segment))
 
 
 Message = namedtuple("Message", "subject payload")
@@ -133,6 +145,15 @@ Request = namedtuple("Request", "subject inbox payload")
 Response = namedtuple("Response", "payload")
 
 
+def one(*, subject, host="0.0.0.0", port=4222, name="one"):
+    """returns one (the first) message received on a subject"""
+    with Client(host=host, port=port, name=name) as client:
+        client.subscribe(subject=subject)
+        while True:
+            for message in client.get():
+                return message
+
+
 if __name__ == "__main__":
     import datetime as dt
     import time
